@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2008 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2009 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -39,13 +39,20 @@ namespace gloox
        * @param cdh A CompressionDataHandler-derived object that will be notified
        * about finished de/compression.
        */
-      CompressionBase( CompressionDataHandler *cdh ) : m_handler( cdh ), m_valid( false ) {}
+      CompressionBase( CompressionDataHandler* cdh ) : m_handler( cdh ), m_valid( false ) {}
 
       /**
        * Virtual Destructor.
        */
       virtual ~CompressionBase() {}
 
+      /**
+       * This function initializes the compression module.
+       * it is mandatory to be called.
+       * @return @b True if the module was initialized successfully, false otherwise.
+       */
+      virtual bool init() = 0;
+
       /**
        * Compresses the given chunk of data.
        * @param data The original (uncompressed) data.
@@ -58,8 +65,17 @@ namespace gloox
        */
       virtual void decompress( const std::string& data ) = 0;
 
+      /**
+       * Performs internal cleanup.
+       * @since 1.0
+       */
+      virtual void cleanup() = 0;
+
     protected:
-      CompressionDataHandler *m_handler;
+      /** A handler for compressed/uncompressed data. */
+      CompressionDataHandler* m_handler;
+
+      /** Whether the compression module can be used. */
       bool m_valid;
 
   };
