@@ -4,6 +4,7 @@ using namespace gloox;
 #include <stdio.h>
 #include <locale.h>
 #include <string>
+#include <cstdio> // [s]print[f]
 
 int main( int /*argc*/, char** /*argv*/ )
 {
@@ -43,7 +44,7 @@ int main( int /*argc*/, char** /*argv*/ )
 
   if( fail == 0 )
   {
-    printf( "Base64: all tests passed\n" );
+    printf( "Base64: OK\n" );
     return 0;
   }
   else
