@@ -1,13 +1,17 @@
+#define GLOOX_TESTS
 #include "../../gloox.h"
 #include "../../jid.h"
 #include "../../dataform.h"
 #include "../../stanza.h"
 #include "../../tag.h"
 #include "../../iqhandler.h"
+#include "../../iq.h"
+#include "../../stanzaextension.h"
 
 #include <stdio.h>
 #include <locale.h>
 #include <string>
+#include <cstdio> // [s]print[f]
 
 const std::string& g_dir = "test.dir";
 const std::string& g_inst = "the instructions";
@@ -15,20 +19,33 @@ const std::string& g_inst = "the instructions";
 namespace gloox
 {
 
+  class Disco;
+  class Capabilities : public StanzaExtension
+  {
+    public:
+      Capabilities() : StanzaExtension( ExtUser + 1 ) {}
+      const std::string& ver() const { return EmptyString; }
+      const std::string& node() const { return EmptyString; }
+  };
+
   class ClientBase
   {
     public:
       ClientBase() {}
       virtual ~ClientBase() {}
       const std::string getID() { return "id"; }
-      virtual void send( Tag *tag ) = 0;
+      virtual void send( IQ& iq, IqHandler*, int ) = 0;
       virtual void trackID( IqHandler *ih, const std::string& id, int context ) = 0;
       void removeIDHandler( IqHandler* ) {}
+      void registerStanzaExtension( StanzaExtension* ext ) { delete ext; }
+      void removeStanzaExtension( int ) {}
   };
 
 }
 
+#define CAPABILITIES_H__
 #define CLIENTBASE_H__
+#define SEARCH_TEST
 #include "../../search.h"
 #include "../../search.cpp"
 #include "../../searchhandler.h"
@@ -47,15 +64,13 @@ class SearchTest : public gloox::SearchHandler, public gloox::ClientBase
       if( directory.full() == g_dir && instructions == g_inst && fields == 15 )
         m_result = true;
     }
-    virtual void handleSearchFields( const gloox::JID& directory, gloox::DataForm *form )
+    virtual void handleSearchFields( const gloox::JID& directory, const gloox::DataForm* form )
     {
       if( m_test != 6 )
         return;
 
       if( directory.full() == g_dir && form != 0 )
         m_result = true;
-
-      delete form;
     }
     virtual void handleSearchResult( const gloox::JID& directory, const gloox::SearchResultList& resultList )
     {
@@ -65,8 +80,10 @@ class SearchTest : public gloox::SearchHandler, public gloox::ClientBase
         {
           gloox::SearchResultList::const_iterator it = resultList.begin();
           if( directory.full() == g_dir && resultList.size() == 2
-              && (*it).first == "f1" && (*it).last == "l1" && (*it).nick == "n1" && (*it).email == "e1"
-              && (*++it).first == "f2" && (*it).last == "l2" && (*it).nick == "n2" && (*it).email == "e2" )
+              && (*it)->first() == "f1" && (*it)->last() == "l1" && (*it)->nick() == "n1"
+              && (*it)->email() == "e1"
+              && (*++it)->first() == "f2" && (*it)->last() == "l2" && (*it)->nick() == "n2"
+              && (*it)->email() == "e2" )
             m_result = true;
           break;
         }
@@ -78,19 +95,22 @@ class SearchTest : public gloox::SearchHandler, public gloox::ClientBase
           break;
       }
     }
-    virtual void handleSearchResult( const gloox::JID& directory, const gloox::DataForm *form )
+    virtual void handleSearchResult( const gloox::JID& directory, const gloox::DataForm* form )
     {
       if( m_test != 8 )
         return;
 
       if( directory.full() == g_dir && form != 0 )
         m_result = true;
-
-      delete form;
     }
-    virtual void handleSearchError( const gloox::JID& /*directory*/, gloox::Stanza* /*stanza*/ ) {}
-    virtual void send( gloox::Tag* tag )
+    virtual void handleSearchError( const gloox::JID& /*directory*/, const gloox::Error* /*error*/ ) {}
+    virtual void send( gloox::IQ& iq, gloox::IqHandler*, int context )
     {
+      m_context = context;
+      gloox::Tag* tag = iq.tag();
+      if( !tag->hasAttribute( "id" ) )
+        tag->addAttribute( "id", "id" );
+
       switch( m_test )
       {
         case 1:
@@ -128,11 +148,10 @@ class SearchTest : public gloox::SearchHandler, public gloox::ClientBase
     void setTest( int test ) { m_test = test; }
     void fetchSearchFields() { m_search.fetchSearchFields( g_dir, this ); }
     bool result() { bool t = m_result; m_result = false; return t; }
-    void feed( gloox::Stanza *s ) { m_search.handleIqID( s, m_context ); }
-    virtual void trackID( gloox::IqHandler* /*ih*/, const std::string& /*id*/, int context )
-      { m_context = context; }
+    void feed( gloox::IQ& s ) { m_search.handleIqID( s, m_context ); }
+    virtual void trackID( gloox::IqHandler* /*ih*/, const std::string& /*id*/, int /*context*/ ) {}
     void search( const gloox::SearchFieldStruct& fields ) { m_search.search( g_dir, 15, fields, this ); }
-    void search( const gloox::DataForm& form ) { m_search.search( g_dir, form, this ); }
+    void search( gloox::DataForm* form ) { m_search.search( g_dir, form, this ); }
   private:
     gloox::Search m_search;
     int m_test;
@@ -160,38 +179,28 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "receive fields (old-style)";
-  gloox::Stanza *iq = new gloox::Stanza( "iq" );
-  iq->addAttribute( "from", g_dir );
-  iq->addAttribute( "to", "searchtest" );
-  iq->addAttribute( "id", "id" );
-  iq->addAttribute( "type", "result" );
-  gloox::Tag *q = new gloox::Tag( iq, "query" );
-  q->addAttribute( "xmlns", gloox::XMLNS_SEARCH );
-  new gloox::Tag( q, "instructions", g_inst );
-  new gloox::Tag( q, "first" );
-  new gloox::Tag( q, "last" );
-  new gloox::Tag( q, "nick" );
-  new gloox::Tag( q, "email" );
-  iq->finalize();
-  t.setTest( 2 );
-  t.feed( iq );
-  if( !t.result() )
   {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
+    name = "receive fields (old-style)";
+    gloox::IQ iq( gloox::IQ::Result, gloox::JID( "searchtest" ), "id" );
+    gloox::SearchFieldStruct sfs( "first", "last", "nick", "email" );
+    gloox::Search::Query* sq = new gloox::Search::Query( gloox::SearchFieldFirst | gloox::SearchFieldLast
+        | gloox::SearchFieldEmail | gloox::SearchFieldNick, sfs );
+    sq->m_instructions = g_inst;
+    iq.addExtension( sq );
+    iq.setFrom( gloox::JID( g_dir ) );
+    t.setTest( 2 );
+    t.feed( iq );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
   }
-  delete iq;
-  iq = 0;
 
   // -------
   name = "search request (old-style)";
   t.setTest( 3 );
-  gloox::SearchFieldStruct sf;
-  sf.first = "first";
-  sf.last = "last";
-  sf.nick = "nick";
-  sf.email = "email";
+  gloox::SearchFieldStruct sf( "first", "last", "nick", "email" );
   t.search( sf );
   if( !t.result() )
   {
@@ -200,68 +209,62 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "search result (old-style)";
-  iq = new gloox::Stanza( "iq" );
-  iq->addAttribute( "from", g_dir );
-  iq->addAttribute( "to", "searchtest" );
-  iq->addAttribute( "id", "id" );
-  iq->addAttribute( "type", "result" );
-  q = new gloox::Tag( iq, "query" );
-  q->addAttribute( "xmlns", gloox::XMLNS_SEARCH );
-  gloox::Tag *i = new gloox::Tag( q,"item" );
-  new gloox::Tag( i, "first", "f1" );
-  new gloox::Tag( i, "last", "l1" );
-  new gloox::Tag( i, "nick", "n1" );
-  new gloox::Tag( i, "email", "e1" );
-  i = new gloox::Tag( q, "item" );
-  new gloox::Tag( i, "first", "f2" );
-  new gloox::Tag( i, "last", "l2" );
-  new gloox::Tag( i, "nick", "n2" );
-  new gloox::Tag( i, "email", "e2" );
-  iq->finalize();
-  t.setTest( 4 );
-  t.feed( iq );
-  if( !t.result() )
   {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
-  }
-  delete iq;
-  iq = 0;
-
-  // -------
-  name = "intermediary search request (old-style)";
-  t.setTest( 3 );
-  sf.first = "first";
-  sf.last = "last";
-  sf.nick = "nick";
-  sf.email = "email";
-  t.search( sf );
-  if( !t.result() )
-  {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
+    name = "search result (old-style)";
+    gloox::IQ iq( gloox::IQ::Result, gloox::JID( "searchtest" ), "id" );
+    iq.setFrom( gloox::JID( g_dir ) );
+    gloox::Tag *q = new gloox::Tag( "query" );
+    q->setXmlns( gloox::XMLNS_SEARCH );
+    gloox::Tag *i = new gloox::Tag( q, "item" );
+    i->addAttribute( "jid", "foo@bar" );
+    new gloox::Tag( i, "first", "f1" );
+    new gloox::Tag( i, "last", "l1" );
+    new gloox::Tag( i, "nick", "n1" );
+    new gloox::Tag( i, "email", "e1" );
+    i = new gloox::Tag( q, "item" );
+    i->addAttribute( "jid", "foo@bar2" );
+    new gloox::Tag( i, "first", "f2" );
+    new gloox::Tag( i, "last", "l2" );
+    new gloox::Tag( i, "nick", "n2" );
+    new gloox::Tag( i, "email", "e2" );
+    iq.addExtension( new gloox::Search::Query( q ) );
+    t.setTest( 4 );
+    t.feed( iq );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
+    delete q;
   }
 
   // -------
-  name = "search result (old-style), empty";
-  iq = new gloox::Stanza( "iq" );
-  iq->addAttribute( "from", g_dir );
-  iq->addAttribute( "to", "searchtest" );
-  iq->addAttribute( "id", "id" );
-  iq->addAttribute( "type", "result" );
-  q = new gloox::Tag( iq, "query" );
-  q->addAttribute( "xmlns", gloox::XMLNS_SEARCH );
-  iq->finalize();
-  t.setTest( 5 );
-  t.feed( iq );
-  if( !t.result() )
   {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
+    name = "intermediary search request (old-style)";
+    t.setTest( 3 );
+    gloox::SearchFieldStruct sf( "first", "last", "nick", "email" );
+    t.search( sf );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
+  }
+
+  // -------
+  {
+    name = "search result (old-style), empty";
+    gloox::IQ iq( gloox::IQ::Result, gloox::JID( "searchtest" ), "id" );
+    iq.addExtension( new gloox::Search::Query() );
+    iq.setFrom( gloox::JID( g_dir ) );
+    t.setTest( 5 );
+    t.feed( iq );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
   }
-  delete iq;
-  iq = 0;
 
   // -------
   name = "fetch fields (dataform)";
@@ -274,30 +277,24 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "receive fields (dataform)";
-  iq = new gloox::Stanza( "iq" );
-  iq->addAttribute( "from", g_dir );
-  iq->addAttribute( "to", "searchtest" );
-  iq->addAttribute( "id", "id" );
-  iq->addAttribute( "type", "result" );
-  q = new gloox::Tag( iq, "query" );
-  q->addAttribute( "xmlns", gloox::XMLNS_SEARCH );
-  gloox::DataForm df( gloox::DataForm::FormTypeForm );
-  q->addChild( df.tag() );
-  iq->finalize();
-  t.setTest( 6 );
-  t.feed( iq );
-  if( !t.result() )
   {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
+    name = "receive fields (dataform)";
+    gloox::IQ iq( gloox::IQ::Result, gloox::JID( "searchtest" ), "id" );
+    iq.setFrom( gloox::JID( g_dir ) );
+    iq.addExtension( new gloox::Search::Query( new gloox::DataForm( gloox::TypeForm ) ) );
+    t.setTest( 6 );
+    t.feed( iq );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
   }
-  delete iq;
-  iq = 0;
 
   // -------
   name = "search request (dataform)";
   t.setTest( 7 );
+  gloox::DataForm* df = new gloox::DataForm( gloox::TypeForm );
   t.search( df );
   if( !t.result() )
   {
@@ -306,26 +303,19 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "search result (dataform)";
-  iq = new gloox::Stanza( "iq" );
-  iq->addAttribute( "from", g_dir );
-  iq->addAttribute( "to", "searchtest" );
-  iq->addAttribute( "id", "id" );
-  iq->addAttribute( "type", "result" );
-  q = new gloox::Tag( iq, "query" );
-  q->addAttribute( "xmlns", gloox::XMLNS_SEARCH );
-  gloox::DataForm df2( gloox::DataForm::FormTypeResult );
-  q->addChild( df2.tag() );
-  iq->finalize();
-  t.setTest( 8 );
-  t.feed( iq );
-  if( !t.result() )
   {
-    ++fail;
-    printf( "test '%s' failed\n", name.c_str() );
-  }
-  delete iq;
-  iq = 0;
+    name = "search result (dataform)";
+    gloox::IQ iq( gloox::IQ::Result, gloox::JID( "searchtest" ), "id" );
+    iq.setFrom( gloox::JID( g_dir ) );
+    iq.addExtension( new gloox::Search::Query( new gloox::DataForm( gloox::TypeResult ) ) );
+    t.setTest( 8 );
+    t.feed( iq );
+    if( !t.result() )
+    {
+      ++fail;
+      printf( "test '%s' failed\n", name.c_str() );
+    }
+    }
 
 
 
@@ -335,7 +325,7 @@ int main( int /*argc*/, char** /*argv*/ )
 
   if( fail == 0 )
   {
-    printf( "Search: all tests passed\n" );
+    printf( "Search: OK\n" );
     return 0;
   }
   else
