@@ -8,6 +8,7 @@
 #include <stdio.h>
 #include <locale.h>
 #include <string>
+#include <cstdio> // [s]print[f]
 
 const std::string& g_dir = "test.dir";
 const std::string& g_inst = "the instructions";
@@ -40,16 +41,17 @@ namespace gloox
       ~ClientBase();
       const std::string getID();
       Disco* disco();
-      void send( Tag *tag );
+      void send( IQ& iq, IqHandler* = 0 , int = 0 );
       void trackID( IqHandler *ih, const std::string& id, int context );
-      void registerIqHandler( IqHandler *ih, const std::string& xmlns );
-      void removeIqHandler( const std::string& xmlns );
-      virtual void handleSIRequestResult( const JID& from, const std::string& sid,
-                                          Tag* si, Tag* ptag, Tag* fneg );
-      virtual void handleSIRequestError( Stanza* stanza, const std::string& sid );
-      virtual void handleSIRequest( const JID& from, const std::string& id, const std::string& profile,
-                                    Tag* si, Tag* ptag, Tag* fneg );
+      void registerIqHandler( IqHandler* ih, int exttype );
+      void removeIqHandler( IqHandler* ih, int exttype );
+      void registerStanzaExtension( StanzaExtension* ext );
+      void removeStanzaExtension( int ext );
       void removeIDHandler( IqHandler* ) {}
+      virtual void handleSIRequestResult( const JID& from, const JID& to, const std::string& sid,
+                                          const SIManager::SI& si );
+      virtual void handleSIRequestError( const IQ& iq, const std::string& /*sid*/ );
+      virtual void handleSIRequest( const JID& from, const JID& to, const std::string& id, const SIManager::SI& si );
       void setTest( int test );
       bool ok();
     private:
@@ -61,8 +63,9 @@ namespace gloox
   ClientBase::~ClientBase() { delete m_disco; }
   const std::string ClientBase::getID() { return "id"; }
   Disco* ClientBase::disco() { return m_disco; }
-  void ClientBase::send( Tag *tag )
+  void ClientBase::send( IQ& iq, IqHandler*, int )
   {
+    Tag* tag = iq.tag();
     switch( m_test )
     {
       case 1:
@@ -78,14 +81,15 @@ namespace gloox
     delete tag;
   }
   void ClientBase::trackID( IqHandler* /*ih*/, const std::string& /*id*/, int /*context*/ ) {}
-  void ClientBase::registerIqHandler( IqHandler* /*ih*/, const std::string& /*xmlns*/ ) {}
-  void ClientBase::removeIqHandler( const std::string& /*xmlns*/ ) {}
-  void ClientBase::handleSIRequestResult( const JID& /*from*/, const std::string& /*sid*/,
-                                          Tag* /*si*/, Tag* /*ptag*/, Tag* /*fneg*/ ) {}
-  void ClientBase::handleSIRequestError( Stanza* /*stanza*/, const std::string& /*sid*/ ) {}
-  void ClientBase::handleSIRequest( const JID& /*from*/, const std::string& /*id*/,
-                                    const std::string& /*profile*/,
-                                    Tag* /*si*/, Tag* /*ptag*/, Tag* /*fneg*/ ) {}
+  void ClientBase::removeIqHandler( IqHandler*, int ) {}
+  void ClientBase::registerIqHandler( IqHandler*, int ) {}
+  void ClientBase::registerStanzaExtension( StanzaExtension* se ) { delete se; }
+  void ClientBase::removeStanzaExtension( int ) {}
+  void ClientBase::handleSIRequestResult( const JID& /*from*/, const JID& /*to*/, const std::string& /*sid*/,
+                                          const SIManager::SI& /*si*/ ) {}
+  void ClientBase::handleSIRequestError( const IQ& /*iq*/, const std::string& /*sid*/ ) {}
+  void ClientBase::handleSIRequest( const JID& /*from*/, const JID& /*to*/, const std::string& /*id*/,
+                                    const SIManager::SI& /*si*/ ) {}
   void ClientBase::setTest( int test ) { m_test = test; }
   bool ClientBase::ok() { bool t = m_ok; m_ok = false; return t; }
 }
@@ -137,7 +141,7 @@ int main( int /*argc*/, char** /*argv*/ )
 
   if( fail == 0 )
   {
-    printf( "SIManager: all tests passed\n" );
+    printf( "SIManager: OK\n" );
     return 0;
   }
   else
