@@ -4,14 +4,15 @@ using namespace gloox;
 #include <stdio.h>
 #include <locale.h>
 #include <string>
+#include <cstdio> // [s]print[f]
 
 int fail = 0;
 
-void printResult( const std::string& name, Tag::TagList& result )
+void printResult( const std::string& name, ConstTagList& result )
 {
   printf( ">-- %s --------------------------------------\n", name.c_str() );
   int i = 0;
-  Tag::TagList::const_iterator it = result.begin();
+  ConstTagList::const_iterator it = result.begin();
   for( ; it != result.end(); ++it, ++i )
   {
     printf( "tag #%d: %s\n", i, (*it)->xml().c_str() );
@@ -38,16 +39,16 @@ int main( int /*argc*/, char** /*argv*/ )
   std::string name;
   Tag *aaa = new Tag( "aaa" );
   Tag *bbb = new Tag( aaa, "bbb" ); bbb->addAttribute( "name", "b1" );
-  Tag *ccc = new Tag( aaa, "ccc" );
-  Tag *ddd = new Tag( ccc, "ddd" );
+  Tag *ccc = new Tag( aaa, "ccc" ); ccc->setCData( "abc" );
+  Tag *ddd = new Tag( ccc, "ddd" ); ddd->setCData( "bcd" );
   Tag *eee = new Tag( ccc, "eee" );
   Tag *fff = new Tag( aaa, "fff" );
   Tag *ggg = new Tag( fff, "ggg" );
   Tag *hhh = new Tag( bbb, "hhh" ); hhh->addAttribute( "name", "h1" );
   Tag *iii = new Tag( bbb, "bbb" ); iii->addAttribute( "name", "b2" );
   Tag *jjj = new Tag( hhh, "bbb" ); jjj->addAttribute( "name", "b3" );
-  Tag::TagList result;
-  Tag::TagList::const_iterator it;
+  ConstTagList result;
+  ConstTagList::const_iterator it;
 //   XPathToken *t = 0;
 
 // <aaa>
@@ -292,7 +293,7 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "find TagList: //bbb";
+  name = "find ConstTagList: //bbb";
   result = aaa->findTagList( "//bbb" );
   it = result.begin();
   if( result.size() != 3 || (*it) != bbb || (*++it) != jjj || (*++it) != iii )
@@ -303,7 +304,7 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 
   // -------
-  name = "find TagList: //ggg";
+  name = "find ConstTagList: //ggg";
   result = aaa->findTagList( "//ggg" );
   it = result.begin();
   if( result.size() != 1 || (*it) != ggg )
@@ -592,8 +593,21 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 //   printf( "--------------------------------------------------------------\n" );
 
+
   // -------
-  name = "empty union 3: /cde|/def";
+  name = "union + predicates: //bbb[@name='b1']|//hhh[@name='h1']";
+  result = aaa->findTagList( "//bbb[@name='b1']|//hhh[@name='h1']" );
+  it = result.begin();
+  if( result.size() != 2 || (*it) != bbb || (*++it) != hhh )
+  {
+    ++fail;
+    printResult( name, result );
+    printf( "test '%s' failed\n", name.c_str() );
+  }
+//   printf( "--------------------------------------------------------------\n" );
+
+  // -------
+  name = "empty union 1: /cde|/def";
   result = aaa->findTagList( "/cde|/def" );
   it = result.begin();
   if( result.size() != 0 )
@@ -604,6 +618,7 @@ int main( int /*argc*/, char** /*argv*/ )
   }
 //   printf( "--------------------------------------------------------------\n" );
 
+
   // ---- ~union ----
 
   // -- ~operators --
@@ -843,6 +858,25 @@ int main( int /*argc*/, char** /*argv*/ )
   // -- predicates --
 
 
+  name = "cdata: //ccc";
+  if( aaa->findCData( "//ccc" ) != "abc" )
+  {
+    ++fail;
+    printResult( name, result );
+    printf( "test '%s' failed\n", name.c_str() );
+  }
+//   printf( "--------------------------------------------------------------\n" );
+
+  name = "cdata: //ddd";
+  if( aaa->findCData( "//ddd" ) != "bcd" )
+  {
+    ++fail;
+    printResult( name, result );
+    printf( "test '%s' failed\n", name.c_str() );
+  }
+//   printf( "--------------------------------------------------------------\n" );
+
+
 
 
 
@@ -896,7 +930,7 @@ int main( int /*argc*/, char** /*argv*/ )
 
   if( fail == 0 )
   {
-    printf( "XPath: all tests passed\n" );
+    printf( "XPath: OK\n" );
     return 0;
   }
   else
