@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2008 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2009 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -21,10 +21,9 @@
 namespace gloox
 {
 
-  GnuTLSClientAnon::GnuTLSClientAnon( TLSHandler *th )
+  GnuTLSClientAnon::GnuTLSClientAnon( TLSHandler* th )
     : GnuTLSBase( th )
   {
-    init();
   }
 
   GnuTLSClientAnon::~GnuTLSClientAnon()
@@ -38,7 +37,9 @@ namespace gloox
     init();
   }
 
-  void GnuTLSClientAnon::init()
+  bool GnuTLSClientAnon::init( const std::string&,
+                               const std::string&,
+                               const StringList& )
   {
     const int protocolPriority[] = { GNUTLS_TLS1, 0 };
     const int kxPriority[]       = { GNUTLS_KX_ANON_DH, 0 };
@@ -47,14 +48,14 @@ namespace gloox
     const int compPriority[]     = { GNUTLS_COMP_ZLIB, GNUTLS_COMP_NULL, 0 };
     const int macPriority[]      = { GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0 };
 
-    if( gnutls_global_init() != 0 )
-      return;
+    if( m_initLib && gnutls_global_init() != 0 )
+      return false;
 
     if( gnutls_anon_allocate_client_credentials( &m_anoncred ) < 0 )
-      return;
+      return false;
 
     if( gnutls_init( m_session, GNUTLS_CLIENT ) != 0 )
-      return;
+      return false;
 
     gnutls_protocol_set_priority( *m_session, protocolPriority );
     gnutls_cipher_set_priority( *m_session, cipherPriority );
@@ -66,6 +67,9 @@ namespace gloox
     gnutls_transport_set_ptr( *m_session, (gnutls_transport_ptr_t)this );
     gnutls_transport_set_push_function( *m_session, pushFunc );
     gnutls_transport_set_pull_function( *m_session, pullFunc );
+
+    m_valid = true;
+    return true;
   }
 
   void GnuTLSClientAnon::getCertInfo()
