@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007-2008 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2007-2009 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -17,13 +17,7 @@
 
 #include "tlsgnutlsbase.h"
 
-#ifdef _WIN32
-# include "../config.h.win"
-#elif defined( _WIN32_WCE )
-# include "../config.h.win"
-#else
-# include "config.h"
-#endif
+#include "config.h"
 
 #ifdef HAVE_GNUTLS
 
@@ -38,7 +32,7 @@ namespace gloox
    *
    * You should not need to use this class directly.
    *
-   * @author Jakob Schrter <js@camaya.net>
+   * @author Jakob Schroeter <js@camaya.net>
    * @since 0.9
    */
   class GnuTLSServerAnon : public GnuTLSBase
@@ -48,18 +42,22 @@ namespace gloox
        * Constructor.
        * @param th The TLSHandler to handle TLS-related events.
        */
-      GnuTLSServerAnon( TLSHandler *th );
+      GnuTLSServerAnon( TLSHandler* th );
 
       /**
        * Virtual destructor.
        */
       virtual ~GnuTLSServerAnon();
 
-      // re-implemented from TLSBase
+      // reimplemented from TLSBase
+      virtual bool init( const std::string& clientKey = EmptyString,
+                         const std::string& clientCerts = EmptyString,
+                         const StringList& cacerts = StringList() );
+
+      // reimplemented from TLSBase
       virtual void cleanup();
 
     private:
-      virtual void init();
       virtual void getCertInfo();
       void generateDH();
 
